require 'pry'
require 'konf'
require 'capistrano/all'
require 'capistrano/setup'
require 'capistrano/deploy'
require 'hat/recipes'
require 'hat/stage'
require 'bundler'
module Hat
  class Deployment

    attr_accessor :configuration, :cap, :stage_name

    def initialize stage_name, file_name = nil
      self.configuration = Konf.new(file_name || 'deploy.yml')
      self.stage_name = stage_name || "staging"
      self.cap = Capistrano::Application.new
    end

    def run
      recipes.download
      run_with_bundler(recipes.gemfile) do
        stage.set_defaults(configuration['defaults'])
        stage.define_task(recipes)
        cap.invoke stage.name   # http://capistranorb.com/documentation/advanced-features/capistrano-pure-ruby/
        cap.invoke 'deploy'
      end
    end

    private

    def recipes
      @recipes ||= Recipes.new(configuration['recipes'] || {})
    end

    def stage
      @stage ||= Stage.new(stage_name, configuration['stages'][stage_name] || {}, recipes.list)
    end

    def run_with_bundler(gemfile)
      if File.exists?(gemfile)
        Bundler.with_clean_env do
          ENV['BUNDLE_GEMFILE'] = gemfile
          Bundler.setup
          yield
        end
      else
        yield
      end
    end
  end
end
